from typing import List

from fastapi import APIRouter

from chef.api.common import generic_get_all
from chef.controllers import UnitsController
from chef.schemas import Unit

router = APIRouter()
units = UnitsController()


@router.get("")
async def get_units() -> List[Unit]:
    return await generic_get_all(units)



"""add images

Revision ID: aaa1b50ab63e
Revises: 39659baca9b3
Create Date: 2024-01-29 20:21:43.012903

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'aaa1b50ab63e'
down_revision = '39659baca9b3'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('recipe', sa.Column('thumbnail_image', sa.String(length=100), nullable=True))
    op.add_column('recipe', sa.Column('detail_image', sa.String(length=100), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('recipe', 'detail_image')
    op.drop_column('recipe', 'thumbnail_image')
    # ### end Alembic commands ###



from contextlib import ContextDecorator
from typing import Union

from sqlalchemy import create_engine
from sqlalchemy.orm import Session

from chef.settings import settings

engine = create_engine(settings.database_uri, echo=True)


class MyContext(ContextDecorator):

    session: Union[Session, None]

    def __init__(self) -> None:
        self.session = None

    def __enter__(self):
        print('in enter')
        self.session = Session(engine)
        return self.session

    def __exit__(self, exc_type, exc_val, exc_tb):
        print('in exit')
        self.session.close()


def get_session():
    return MyContext()



from importlib.metadata import version
from fastapi import APIRouter

from chef.api.categories import router as category_router
from chef.api.ingredients import router as ingredient_router
from chef.api.recipes import router as recipe_router
from chef.api.tags import router as tag_router
from chef.api.images import router as image_router
from chef.api.units import router as units_router


VERSION = version("chef-recipes")

api_router = APIRouter(prefix='/api')


@api_router.get("/info")
def get_server_info():
    return {
        "version": VERSION
    }

api_router.include_router(recipe_router, prefix="/recipes")
api_router.include_router(category_router, prefix="/categories")
api_router.include_router(ingredient_router, prefix="/ingredients")
api_router.include_router(tag_router, prefix="/tags")
api_router.include_router(units_router, prefix="/units")
api_router.include_router(image_router, prefix="/images")



"""Add favorite field.

Revision ID: 39659baca9b3
Revises: 998321576878
Create Date: 2023-06-30 14:44:00.330099

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '39659baca9b3'
down_revision = '998321576878'
branch_labels = None
depends_on = None


def upgrade() -> None:
    op.add_column('recipe', sa.Column('favorite', sa.Boolean(), server_default=sa.false()))


def downgrade() -> None:
    op.drop_column('recipe', 'favorite')



import os

from PIL import Image
from chef.settings import settings

from loguru import logger


class Handler:

    THUMB = 96, 96
    SMALL = 540, 540
    MEDIUM = 768, 768
    LANDSCAPE = 1280, 720

    ALL_SIZES = [
        ("thumb", THUMB),
        ("small", SMALL),
        ("medium", MEDIUM)
    ]

    def __init__(self, file, item_id) -> None:
        self.file = file
        self.item_id = item_id
        self.target = settings.images_folder

    def _create_folder(self):
        if not os.path.isdir(self.target):
            os.makedirs(self.target, exist_ok=True)

        recipe_folder = os.path.join(self.target, str(self.item_id))
        if not os.path.isdir(recipe_folder):
            os.mkdir(recipe_folder)

        return recipe_folder

    def _sizes(self):
        return self.ALL_SIZES

    def create_images_set(self):
        image = Image.open(self.file)
        target_dir = self._create_folder()

        for name, size in self._sizes():
            tmp_img = image.copy()
            tmp_img.thumbnail(size, Image.LANCZOS)
            path = os.path.join(target_dir, f"{name}.jpeg")
            logger.info(f"Saving image to '{path}'")
            tmp_img.save(path)


class CategoryHandler(Handler):

    def _create_folder(self):
        if not os.path.isdir(self.target):
            logger.error(f"Images base DIR does not exist: {self.target}")
            raise Exception(f"Images base DIR does not exist: {self.target}")

        categories_folder = os.path.join(self.target, "categories")
        if not os.path.isdir(categories_folder):
            os.mkdir(categories_folder)

        category_folder = os.path.join(categories_folder, str(self.item_id))
        if not os.path.isdir(category_folder):
            os.mkdir(category_folder)

        return category_folder

    def _sizes(self):
        return [
            ("landscape", self.LANDSCAPE)
        ]



"""Rename fibres.

Revision ID: b8b797b55d1b
Revises: f91610b78d55
Create Date: 2021-04-10 13:32:03.840335

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'b8b797b55d1b'
down_revision = 'f91610b78d55'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('fibres', sa.Float(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('fibers')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('fibers', sa.Float(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('fibres')
    # ### end Alembic commands ###



import os.path
from enum import StrEnum
from pathlib import Path
from typing import Union, Optional
from distutils.sysconfig import get_python_lib
from loguru import logger

from pydantic import Field
from pydantic_settings import BaseSettings


HOME_DIR = Path.home() / '.chef'


class StorageType(StrEnum):
    LOCAL = 'LOCAL'


class ImageFormat(StrEnum):
    AVIF = 'AVIF'


def get_bundled_frontend_path():
    site_packages = get_python_lib()
    frontend_root = os.path.join(site_packages, "src", "js", "chef", "dist")
    logger.info(frontend_root)
    if os.path.isdir(frontend_root):
        return frontend_root


def get_locally_build_frontend_path():
    return "./src/js/chef/dist" if os.path.isdir("./src/js/chef/dist") else None


def get_default_frontend_path() -> str:
    return get_bundled_frontend_path() or get_locally_build_frontend_path()


class Settings(BaseSettings):
    """Default config will store all user data in ~/.chef"""
    serve_frontend_path: Union[str, None] = Field(default_factory=get_default_frontend_path)
    serve_frontend: bool = True
    serve_static: bool = True
    database_uri: str = "sqlite:///" + os.path.join(HOME_DIR,  "chef.db")
    log_file: Union[str, None] = None
    log_level: str = "DEBUG"
    log_sql: bool = False

    storage_backend: StorageType = StorageType.LOCAL
    images_folder: str = os.path.join(HOME_DIR,  "images")

    uvicorn_host: str = "0.0.0.0"
    uvicorn_port: int = 8000

    public_url: str = "http://localhost:8000"
    gpt_enpoint_url: str = "http://localhost:8001"

    sentry_dsn: Optional[str] = None


settings = Settings(_env_file=".env")



import os.path
from pathlib import Path

import typer
import uvicorn
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from fastapi.openapi.models import Response
from loguru import logger
from starlette.requests import Request
from starlette.responses import HTMLResponse
from starlette.staticfiles import StaticFiles

from chef.api import api_router
from chef.models import ensure_tables
from chef.scripts.migrate_images import run as migrate_images_fn
from chef.settings import settings, Settings
from migrations.migrate import run as migrate_db_fn

cli_app = typer.Typer()

if settings.sentry_dsn:
    import sentry_sdk
    from sentry_sdk.integrations.fastapi import FastApiIntegration
    from sentry_sdk.integrations.starlette import StarletteIntegration
    sentry_sdk.init(
        dsn=settings.sentry_dsn,
        # Set traces_sample_rate to 1.0 to capture 100%
        # of transactions for performance monitoring.
        traces_sample_rate=1.0,
        # Set profiles_sample_rate to 1.0 to profile 100%
        # of sampled transactions.
        # We recommend adjusting this value in production.
        profiles_sample_rate=1.0,
        integrations=[
            StarletteIntegration(
                transaction_style="endpoint",
                failed_request_status_codes=[403, range(500, 599)],
            ),
            FastApiIntegration(
                transaction_style="endpoint",
                failed_request_status_codes=[403, range(500, 599)],
            ),
        ],
    )


app = FastAPI(
    title="Chef",
    summary="Personal recipe management app.",
    version="2.5.0"
)
app.include_router(api_router)

app.add_middleware(
    CORSMiddleware,
    allow_origins=[
        "http://localhost:5173",
        "http://localhost:8080",
    ],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


def print_banner(settings_in: Settings):
    options = "\n".join([f"{k.upper()}: '{v}'" for k, v in settings_in])
    logger.info(f"""

        ███████╗██╗  ██╗███████╗███████╗
        ██╔════╝██║  ██║██╔════╝██╔════╝
        ██║     ███████║█████╗  █████╗  
        ██║     ██╔══██║██╔══╝  ██╔══╝  
        ║██████ ██║  ██║███████╗██║     
        ╚═════════╝  ╚═╝╚══════╝╚═╝

        ...starting with following settings:

    {options}
    """)


@cli_app.command()
def migrate_images():
    print_banner(settings)
    should_it_continue = input("continue? [y/n]")
    if should_it_continue not in ("n", "N", "no", "No", "NO"):
        migrate_images_fn()


@cli_app.command()
def migrate_db():
    migrate_db_fn()


class StaticFilesCache(StaticFiles):
    def __init__(
            self,
            *args,
            cachecontrol="public, max-age=31536000, s-maxage=31536000, immutable",
            **kwargs
    ):
        self.cachecontrol = cachecontrol
        super().__init__(*args, **kwargs)

    def file_response(self, *args, **kwargs) -> Response:
        resp: Response = super().file_response(*args, **kwargs)
        resp.headers.setdefault("Cache-Control", self.cachecontrol)
        return resp


@cli_app.command()
def serve(hostname: str = settings.uvicorn_host, port: int = settings.uvicorn_port):
    settings.uvicorn_port = port
    settings.uvicorn_host = hostname
    print_banner(settings)

    if not os.path.isdir(settings.images_folder):
        logger.info(f"Creating images folder: {settings.images_folder}")
        os.makedirs(settings.images_folder, exist_ok=True)

    ensure_tables()

    if settings.log_file:
        logger.add(settings.log_file, level=settings.log_level)

    if settings.serve_frontend:
        if not settings.serve_frontend_path or not os.path.isdir(settings.serve_frontend_path):
            logger.error(
                f"Frontend files '{settings.serve_frontend_path}' not found, "
                f"frontend can't be served."
            )
            return
        app.mount("/images", StaticFilesCache(directory=settings.images_folder))
        app.mount("/", StaticFiles(directory=settings.serve_frontend_path, html=True))


    @app.exception_handler(404)
    async def redirect_all_requests_to_frontend(request: Request, exc: HTTPException):
        return HTMLResponse(open(Path(settings.serve_frontend_path) / 'index.html').read())

    uvicorn.run(app, host=hostname, port=port)


def app_entrypoint():
    cli_app()


if __name__ == '__main__':
    app_entrypoint()



from typing import List

from fastapi import APIRouter

from chef.api.common import generic_get, generic_get_all, generic_delete, generic_update, \
    generic_create
from chef.controllers import CategoriesController
from chef.schemas import Category, CreateOrUpdateCategory

router = APIRouter()
categories = CategoriesController()


@router.get("")
async def get_categories() -> List[Category]:
    return await generic_get_all(categories)


@router.post("", status_code=201)
async def create_category(create_data: CreateOrUpdateCategory) -> Category:
    return await generic_create(categories, create_data)


@router.get("/{item_id}")
async def get_category(item_id: int) -> Category:
    return await generic_get(categories, item_id)


@router.delete("/{item_id}")
async def delete_category(item_id: int) -> None:
    return await generic_delete(categories, item_id)


@router.put('/{item_id}')
async def update_category(item_id: int, update_data: CreateOrUpdateCategory) -> Category:
    return await generic_update(categories, item_id, update_data)



import os.path

import pillow_avif
from PIL import Image
from loguru import logger

from chef.image.storage import StorageProtocol, LocalStorage
from chef.settings import StorageType, ImageFormat

StorageBackends: dict[StorageType, StorageProtocol] = {
    StorageType.LOCAL: LocalStorage(),
    # implement more...
}


SIZES = {
    "thumbnail": (250, 250),
    "small": (476, 476),
    # "medium": (768, 768),
}


def _compress_and_store_avif(
        fp: any,
        image_format: ImageFormat,
        image_size: tuple[int, int],
        storage_backend: StorageType = StorageType.LOCAL
) -> str:
    image = Image.open(fp)
    image.thumbnail(image_size)
    logger.debug(f"Image shrunk to {image_size}")
    image_url = StorageBackends.get(storage_backend).store(image, image_format)
    logger.debug(f"Image stored ({storage_backend}) at: {image_url}")
    public_url = os.path.join("/images", image_url.split("/")[-1])
    logger.debug(f"Image available at: {public_url}")
    return public_url


def compress_and_store(
        fp: any,
        image_format: ImageFormat = ImageFormat.AVIF,
        is_thumbnail: bool = False,
) -> str:
    target_size = SIZES.get("thumbnail" if is_thumbnail else "small")

    if image_format == ImageFormat.AVIF:
        return _compress_and_store_avif(fp, ImageFormat.AVIF, target_size)

    msg = f"Unsupported image format: {image_format}"
    logger.error(msg)
    raise Exception(msg)



"""Add category model.

Revision ID: 998321576878
Revises: b8b797b55d1b
Create Date: 2021-04-21 08:31:11.799973

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '998321576878'
down_revision = 'b8b797b55d1b'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('category',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('category_tags',
    sa.Column('category_id', sa.Integer(), nullable=False),
    sa.Column('tag_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['category_id'], ['category.id'], ),
    sa.ForeignKeyConstraint(['tag_id'], ['tag.id'], ),
    sa.PrimaryKeyConstraint('category_id', 'tag_id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('category_tags')
    op.drop_table('category')
    # ### end Alembic commands ###



"""tweak ingredient item

Revision ID: c7b802139fbf
Revises: aaa1b50ab63e
Create Date: 2024-06-24 15:03:02.852874

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'c7b802139fbf'
down_revision = 'aaa1b50ab63e'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###



"""add ingredient item order

Revision ID: ea9e0ff5a7ff
Revises: c7b802139fbf
Create Date: 2024-09-23 20:07:17.783503

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'ea9e0ff5a7ff'
down_revision = 'c7b802139fbf'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient_item', sa.Column('order', sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('ingredient_item', 'order')
    # ### end Alembic commands ###



from datetime import datetime
from uuid import UUID, uuid4

from pydantic import BaseModel, validator, Field, ConfigDict

from typing import List, Union, Optional

from chef.models import Recipe as RecipeDb
from chef.models import Category as CategoryDb
from chef.models import Tag as TagDb
from chef.models import IngredientItem as IngredientItemDb
from chef.models import Ingredient as IngredientDb
from chef.models import Unit as UnitDb


class Base(BaseModel):
    id: int

    class Meta:
        orm_model = None

    @staticmethod
    def get_excluded_fields():
        return []

class Named(Base):
    name: str


# ------- Unit ------- #

class Unit(Named):
    id: Union[int, None] = None
    grams: float = 0  # todo: how many grams in this unit - doesn't make sense

    class Meta:
        orm_model = UnitDb

# ------- Tag ------- #


class Tag(Named):

    class Meta:
        orm_model = TagDb


class UpdateTag(BaseModel):
    id: Union[int, None] = None
    name: str


# ------- Category ------- #

class Category(Named):
    tags: List[Tag] = Field(default_factory=list)

    class Meta:
        orm_model = CategoryDb


class CreateOrUpdateCategory(BaseModel):
    tags: List[Base] = Field(default_factory=list)
    name: str


# ------- Ingredient ------- #

class Ingredient(Named):
    approx_per_piece: float = 1  # g / piece

    energy: float = 0.0  # kcal
    fats: float = 0.0  # g / 100g
    carbs: float = 0.0  # g / 100g
    proteins: float = 0.0  # g / 100g
    fibres: float = 0.0  # g / 100g
    salt: float = 0.0  # g / 100g

    is_liquid: bool = False
    density: float = 1000.0  # g / L

    @staticmethod
    def get_excluded_fields():
        return ["uuid"]

    class Meta:
        orm_model = IngredientDb


class UpdateIngredient(Ingredient):
    """Update list of ingredients during a recipe create op...

    ...if 'id' is missing, new ingredient shall be created.
    """
    id: Union[int, None] = None


class IngredientItemBase(BaseModel):
    """
    -> [included in shopping list] <amount> <unit> <ingredient> (<note>)
    -> [x] 400 g tomatoes (roughly chopped)
    """
    ingredient: Ingredient
    amount: float
    unit: Unit
    order: Optional[int] = None
    uuid: UUID = Field(default_factory=uuid4)

    note: Union[str, None] = None
    exclude: bool = False  # todo: remove?

    model_config = ConfigDict(coerce_numbers_to_str=True)


class IngredientItem(IngredientItemBase, Base):
    """
    -> [included in shopping list] <amount> <unit> <ingredient> (<note>)
    -> [x] 400 g tomatoes (roughly chopped)
    """
    class Meta:
        orm_model = IngredientItemDb
        exclude_items = ["uuid"]


class CreateOrUpdateIngredientItem(IngredientItemBase, BaseModel):
    id: Union[int, None] = None
    ingredient: UpdateIngredient
    unit: Unit = Unit(name="pcs")
    model_config = ConfigDict(extra='allow')


# ------- Recipe ------- #

class BaseRecipe(BaseModel):
    subtitle: Union[str, None] = None
    source_name: Union[str, None] = None
    source: Union[str, None] = None
    draft: bool = False
    favorite: bool = False
    portions: int = 4
    tags: List[Tag] = []


class RecipeDetail(BaseRecipe):
    subtitle: Union[str, None] = None
    source_name: Union[str, None] = None
    source: Union[str, None] = None
    draft: bool = False
    favorite: bool = False
    portions: int = 4
    ingredients: List[IngredientItem] = Field(default_factory=list)
    tags: List[Tag] = []

    # HTML content + rich text editor on the frontend
    body: Union[str, None] = None


class CreateOrUpdateRecipe(RecipeDetail, BaseModel):
    title: str  # mandatory
    ingredients: List[CreateOrUpdateIngredientItem] = Field(default_factory=list)
    tags: List[UpdateTag] = Field(default_factory=list)


class Recipe(Base, RecipeDetail):
    title: str
    thumbnail_image: Union[str, None]
    detail_image: Union[str, None]
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    class Meta:
        orm_model = RecipeDb


class RecipeListItem(Base):
    title: str
    thumbnail_image: Union[str, None]
    detail_image: Union[str, None]
    tags: List[Tag] = []



from pathlib import Path
from typing import Protocol

from uuid import uuid4

from PIL.Image import Image

from chef.settings import settings, ImageFormat


class StorageProtocol(Protocol):

    def store(self, image: Image, image_format: ImageFormat) -> str:
        ...


class LocalStorage:

    system_path: Path
    image_quality: int

    def __init__(self, system_path: Path = None, image_quality: int = None):
        self.system_path = system_path or Path(settings.images_folder)
        self.image_quality = image_quality or 70

    def store(self, image: Image, image_format: ImageFormat) -> str:
        final_path = self.system_path / f"{uuid4()}.{image_format.lower()}"
        image.save(final_path, quality=self.image_quality, save_all=True)
        return str(final_path)



"""Add Timestamps

Revision ID: e454e6860914
Revises: ea9e0ff5a7ff
Create Date: 2024-12-03 11:48:10.519678

"""
from datetime import datetime

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = 'e454e6860914'
down_revision = 'ea9e0ff5a7ff'
branch_labels = None
depends_on = None

tables = ("category", "ingredient", "ingredient_item", "recipe", "tag", "unit")
columns = ("created_at", "updated_at")

def upgrade() -> None:
    # ADD COLS
    for table in tables:
        with op.batch_alter_table(table) as batch_op:
            for col in columns:
                batch_op.add_column(sa.Column(col, sa.DateTime(), nullable=True))

    # DATA MIGRATION
    for table in tables:
        set_clause = ", ".join([f"{col} = '{datetime.utcnow()}'" for col in columns])
        op.execute(f"UPDATE {table} SET {set_clause}")

    # SET TO NONNULL
    for table in tables:
        with op.batch_alter_table(table) as batch_op:
            for col in columns:
                batch_op.alter_column(col, nullable=False)


def downgrade() -> None:
    for table in tables:
        with op.batch_alter_table(table) as batch_op:
            for col in columns:
                batch_op.drop_column(col)



import typing
from datetime import datetime
from typing import List, Generic, TypeVar, Union, Type, Optional

from fastapi import HTTPException
from pydantic import BaseModel
from sqlalchemy import select, update
from sqlalchemy.orm import Session

from chef.models import Category as CategoryDb, Recipe as RecipeDb
from chef.schemas import Recipe, Tag, Category, Ingredient, CreateOrUpdateRecipe, \
    IngredientItem, Unit, UpdateIngredient, CreateOrUpdateCategory

from chef.image.thumbnailer import compress_and_store
from chef.settings import settings

C = TypeVar('C', bound=BaseModel)
R = TypeVar('R', bound=BaseModel)
U = TypeVar('U', bound=BaseModel)


class Controller(Generic[C, R, U]):

    @property
    def create_schema(self) -> Type[C]:
        return typing.get_args(self.__orig_bases__[0])[0]

    @property
    def read_schema(self) -> Type[R]:
        return typing.get_args(self.__orig_bases__[0])[1]

    @property
    def orm(self):
        return self.read_schema.Meta.orm_model

    @property
    def update_schema(self) -> Type[U]:
        return typing.get_args(self.__orig_bases__[0])[2]

    async def _fill_model(self, session: Session, model, data: Union[C, U]):
        """Override for more complex models."""
        for attr, value in data.dict(exclude_none=True).items():
            if attr in data.get_excluded_fields():
                continue
            setattr(model, attr, value)

    async def get_all(self, session: Session) -> List[R]:
        return [
            self.get_transform(self.read_schema(**item.dictionary))
            for item in session.scalars(select(self.read_schema.Meta.orm_model)).all()
        ]

    def get_transform(self, item: R) -> R:
        return item

    async def get_single_or_none(self, session: Session, item_id: int) -> Optional[R]:
        db_item = session.get(self.read_schema.Meta.orm_model, item_id)
        if db_item:
            return self.get_transform(self.read_schema(**db_item.dictionary))

        return None

    async def get_single(self, session: Session, item_id: int) -> R:
        recipe = await self.get_single_or_none(session, item_id)
        if not recipe:
            raise HTTPException(
                status_code=404, detail=f"{self.read_schema.__class__} id={item_id} not found"
            )
        return recipe

    async def delete_single(self, session: Session, item_id: int) -> None:
        item = session.get(self.read_schema.Meta.orm_model, item_id)
        if not item:
            raise HTTPException(
                status_code=404, detail=f"{self.read_schema.__class__} id={item_id} not found"
            )
        session.delete(item)
        session.commit()

    async def create(self, session: Session, data: C) -> R:
        new_model = self.read_schema.Meta.orm_model()
        await self._fill_model(session, new_model, data)
        session.add(new_model)
        session.commit()
        return await self.get_single(session, item_id=new_model.id)

    async def create_or_update(self, session: Session, data: Union[U, C]) -> R:
        if getattr(data, "id", None):
            item = session.get(self.read_schema.Meta.orm_model, data.id)
        else:
            item = self.read_schema.Meta.orm_model()
        await self._fill_model(session, item, data)
        session.add(item)
        session.commit()
        return self.read_schema(**session.get(self.read_schema.Meta.orm_model, item.id).dictionary)

    async def update(self, session: Session, item_id: int, data: U) -> R:
        item = session.get(self.read_schema.Meta.orm_model, item_id)
        if not item:
            raise HTTPException(
                status_code=404, detail=f"{self.read_schema.__class__} id={data.id} not found"
            )
        await self._fill_model(session, item, data)
        session.add(item)
        session.commit()
        return await self.get_single(session, item.id)


class TagController(Controller[Tag, Tag, Tag]):
    ...


class IngredientsController(Controller[UpdateIngredient, Ingredient, UpdateIngredient]):

    async def delete_single(self, session: Session, item_id: int) -> None:
        ingredient = session.get(self.read_schema.Meta.orm_model, item_id)
        if ingredient.ingredients_items:
            recipes = [
                r.title
                for item in ingredient.ingredients_items
                for r in item.recipes
            ]
            if recipes:
                raise HTTPException(
                    status_code=400,
                    detail=f"Ingredient still attached to some recipes: {', '.join(recipes)}",
                )
        return await super().delete_single(session, item_id)


class CategoriesController(Controller[CreateOrUpdateCategory, Category, CreateOrUpdateCategory]):

    async def _fill_model(self,  session: Session, model: CategoryDb, data: CreateOrUpdateCategory):
        tags = []
        for tag in data.tags:
            item = session.get(Tag.Meta.orm_model, tag.id)
            if not item:
                raise HTTPException(
                    status_code=400, detail=f"Tag id={tag.id} not found"
                )
            tags.append(item)
        for attr, value in data.dict(exclude_none=True, exclude={"tags": True}).items():
            setattr(model, attr, value)
        model.tags = tags


class UnitsController(Controller[Unit, Unit, Unit]):

    async def create_or_update(self, session: Session, data: Union[U, C]) -> R:
        if getattr(data, "id", None):
            item = session.get(self.read_schema.Meta.orm_model, data.id)
        else:
            same_name = session \
                .query(self.read_schema.Meta.orm_model) \
                .filter_by(name=data.name) \
                .first()
            if same_name:
                item = same_name
            else:
                item = self.read_schema.Meta.orm_model()
        for attr in data.dict(exclude={"id": True}):
            setattr(item, attr, getattr(data, attr))
        session.add(item)
        session.commit()
        return self.read_schema(
            **session.query(self.read_schema.Meta.orm_model)
            .filter_by(name=item.name)
            .first()
            .dictionary
        )


class RecipesController(Controller[CreateOrUpdateRecipe, Recipe, CreateOrUpdateRecipe]):

    def get_transform(self, item: Recipe) -> Recipe:
        if item.detail_image:
            item.detail_image = settings.public_url + item.detail_image

        if item.thumbnail_image:
            item.thumbnail_image = settings.public_url + item.thumbnail_image

        return item

    @staticmethod
    async def _get_ingredients_and_tags(session: Session, data: CreateOrUpdateRecipe):
        """Load (or create and load) entire models and for specified ingredients and tags."""
        updated_ingredients = []
        ingredients_controller = IngredientsController()
        units_controller = UnitsController()
        for ingredient_item in data.ingredients:
            updated_item = None
            if ingredient_item.id:
                updated_item = session.get(IngredientItem.Meta.orm_model, ingredient_item.id)
                for attr, value in ingredient_item.dict(
                        exclude_none=True,
                        exclude={"unit": True, "ingredient": True, 'id': True, 'uuid': True}
                ).items():
                    setattr(updated_item, attr, value)

            if not updated_item:
                updated_item = IngredientItem.Meta.orm_model(
                    **ingredient_item.dict(
                        exclude={'ingredient': True, 'unit': True, 'id': True, 'uuid': True}
                    )
                )
            u = await units_controller.create_or_update(session, ingredient_item.unit)
            updated_item.unit_id = u.id
            i = await ingredients_controller.create_or_update(session, ingredient_item.ingredient)
            updated_item.ingredient_id = i.id
            updated_ingredients.append(updated_item)

        updated_tags = []
        for tag in data.tags:
            if getattr(tag, "id", None):
                updated_tags.append(session.get(Tag.Meta.orm_model, tag.id))
            else:
                updated_tags.append(
                    Tag.Meta.orm_model(**tag.dict())
                )
        return updated_ingredients, updated_tags

    async def _fill_model(self, session: Session, model, data: Union[C, U]):
        if not model:
            raise HTTPException(
                status_code=404, detail=f"{self.read_schema.__class__} id={data.id} not found"
            )
        for attr, value in data.dict(
                exclude_none=True, exclude={"tags": True, "ingredients": True}
        ).items():
            setattr(model, attr, value)
        updated_ingredients, updated_tags = await self._get_ingredients_and_tags(session, data)
        model.ingredients = updated_ingredients
        model.tags = updated_tags

    async def get_by_category(self, session: Session, category_id: int = None) -> List[Recipe]:
        cat = session.scalars(
            select(Category.Meta.orm_model)
            .filter_by(id=category_id)
            .limit(1)
        ).first()
        if cat:
            result = [recipe for tag in cat.tags for recipe in tag.recipes]
            return [self.read_schema(**r.dictionary) for r in result]
        return []

    async def get_all_and_filter(self, session: Session, **filters: typing.Any) -> List[Recipe]:
        recipes = session.scalars(
            select(self.read_schema.Meta.orm_model)
            .filter_by(**filters)
        ).all()
        return [self.read_schema(**r.dictionary) for r in recipes]

    async def get_all_since(self, session: Session, since: datetime) -> List[Recipe]:
        recipes = session.scalars(
            select(self.read_schema.Meta.orm_model)
            .filter(RecipeDb.updated_at >= since)
        ).all()
        return [self.read_schema(**r.dictionary) for r in recipes]

    async def create_or_update(self, session: Session, data: Union[U, C]) -> R:
        raise ValueError("Not allowed on this controller!")

    def update_thumbnail(self, session: Session, recipe_id: int, image: any):
        url = compress_and_store(image, is_thumbnail=True)
        # recipe = (
        #     session
        #     .query(self.read_schema.Meta.orm_model)
        #     .filter_by(id=recipe_id)
        #     .first()
        # )
        # recipe.thumbnail_image = url
        # session.add(recipe)
        # try:
        #     session.commit()
        # except:
        #     session.rollback()
        #     raise
        stmt = (
            update(self.read_schema.Meta.orm_model)
            .where(self.read_schema.Meta.orm_model.id == recipe_id)
            .values(thumbnail_image=url)
        )
        session.execute(statement=stmt)

    def update_image(self, session: Session, recipe_id: int, image: any):
        url = compress_and_store(image, is_thumbnail=False)
        # recipe = (
        #     session
        #     .query(self.read_schema.Meta.orm_model)
        #     .filter_by(id=recipe_id)
        #     .first()
        # )
        # recipe.detail_image = url
        # session.add(recipe)
        # try:
        #     session.commit()
        # except:
        #     session.rollback()
        #     raise
        stmt = (
            update(self.read_schema.Meta.orm_model)
            .where(self.read_schema.Meta.orm_model.id == recipe_id)
            .values(detail_image=url)
        )
        session.execute(statement=stmt)



from typing import TypeVar, List

from pydantic import BaseModel
from sqlalchemy.orm import Session

from chef.controllers import Controller
from chef.models import engine

C = TypeVar('C', bound=BaseModel)
R = TypeVar('R', bound=BaseModel)
U = TypeVar('U', bound=BaseModel)


async def generic_get(controller: Controller[C, R, U], item_id: int) -> R:
    with Session(engine()) as session:
        return await controller.get_single(session, item_id)


async def generic_get_all(controller: Controller[C, R, U]) -> List[R]:
    with Session(engine()) as session:
        return await controller.get_all(session)


async def generic_create(controller: Controller[C, R, U], create_data: C) -> R:
    with Session(engine()) as session:
        return await controller.create(session, create_data)


async def generic_delete(controller: Controller[C, R, U], item_id: int) -> None:
    with Session(engine()) as session:
        return await controller.delete_single(session, item_id)


async def generic_update(controller: Controller[C, R, U], item_id: int, data: U) -> R:
    with Session(engine()) as session:
        return await controller.update(session, item_id, data)



"""Initial DB schema.

Revision ID: 3251a114750c
Revises:
Create Date: 2021-04-07 15:26:38.532164

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '3251a114750c'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('ingredient',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=False),
    sa.Column('approx_per_piece', sa.Float(), nullable=True),
    sa.Column('energy', sa.Float(), nullable=True),
    sa.Column('fats', sa.Float(), nullable=True),
    sa.Column('carbs', sa.Float(), nullable=True),
    sa.Column('protein', sa.Float(), nullable=True),
    sa.Column('salt', sa.Float(), nullable=True),
    sa.Column('is_liquid', sa.Boolean(), nullable=True),
    sa.Column('density', sa.Float(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('recipe',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('title', sa.String(length=80), nullable=False),
    sa.Column('subtitle', sa.String(length=50), nullable=True),
    sa.Column('source_name', sa.String(length=100), nullable=True),
    sa.Column('source', sa.String(length=100), nullable=True),
    sa.Column('draft', sa.Boolean(), nullable=True),
    sa.Column('portions', sa.Integer(), nullable=True),
    sa.Column('body', sa.String(length=1000), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('tag',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('unit',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=80), nullable=False),
    sa.Column('grams', sa.Float(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name')
    )
    op.create_table('ingredient_item',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('ingredient_id', sa.Integer(), nullable=True),
    sa.Column('amount', sa.Float(), nullable=True),
    sa.Column('unit_id', sa.Integer(), nullable=True),
    sa.Column('note', sa.String(length=10), nullable=True),
    sa.Column('exclude', sa.Boolean(), nullable=True),
    sa.ForeignKeyConstraint(['ingredient_id'], ['ingredient.id'], ),
    sa.ForeignKeyConstraint(['unit_id'], ['unit.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('tags',
    sa.Column('recipe_id', sa.Integer(), nullable=False),
    sa.Column('tag_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['recipe_id'], ['recipe.id'], ),
    sa.ForeignKeyConstraint(['tag_id'], ['tag.id'], ),
    sa.PrimaryKeyConstraint('recipe_id', 'tag_id')
    )
    op.create_table('ingredients',
    sa.Column('recipe_id', sa.Integer(), nullable=False),
    sa.Column('ingredient_item_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['ingredient_item_id'], ['ingredient_item.id'], ),
    sa.ForeignKeyConstraint(['recipe_id'], ['recipe.id'], ),
    sa.PrimaryKeyConstraint('recipe_id', 'ingredient_item_id')
    )
    # ### end Alembic commands ###



from typing import List

from sqlalchemy import Column, ForeignKey, Integer, String, Boolean, Float, Table, create_engine, \
    Engine, DateTime, func
from sqlalchemy.orm import mapped_column, relationship, DeclarativeBase, Mapped

from chef.settings import settings


def engine() -> Engine:
    return create_engine(settings.database_uri, echo=settings.log_sql)


def ensure_tables():
    Base.metadata.create_all(engine())


def _dictify(obj, depth=100):
    if obj is None:
        return None
    if issubclass(obj.__class__, Base) and depth > 0:
        return obj.get_dictionary(depth=depth - 1)
    if issubclass(obj.__class__, list):
        if depth > 1:
            return [_dictify(o, depth=depth - 1) for o in obj]
        else:
            return [str(o) for o in obj]
    if isinstance(obj, int) or isinstance(obj, float):
        return obj
    try:
        try:
            return int(obj)
        except (TypeError, ValueError):
            pass
        return float(obj)
    except (TypeError, ValueError, SyntaxError):
        pass
    return str(obj)


class Base(DeclarativeBase):
    id: Mapped[int] = mapped_column(primary_key=True)
    created_at = Column(DateTime, default=func.now(), server_default=func.now())
    updated_at = Column(DateTime, default=func.now(), onupdate=func.now(), server_onupdate=func.now())

    def _get_attributes(self, exclude: List[str] = None) -> dict:
        """Get model attributes to be parsed into JSON response"""
        if not exclude:
            exclude = []

        result = {}
        attributes = getattr(self, '__items__', [])
        for attr in attributes:
            if attr in exclude:
                continue
            try:
                result[attr] = getattr(self, attr)
            except AttributeError:
                continue
        return result

    def get_dictionary(self, depth=99, exclude=None):
        """Serialize recursively according to the __items__ attribute."""
        return {
            k: _dictify(v, depth=depth) for k, v in self._get_attributes(exclude=exclude).items()
        }

    @property
    def dictionary(self):
        return self.get_dictionary()


ingredients = Table(
    'ingredients',
    Base.metadata,
    Column("recipe_id", Integer, ForeignKey("recipe.id"), primary_key=True),
    Column("ingredient_item_id", Integer, ForeignKey("ingredient_item.id"), primary_key=True)
)

tags = Table(
    'tags',
    Base.metadata,
    Column("recipe_id", Integer, ForeignKey("recipe.id"), primary_key=True),
    Column("tag_id", Integer, ForeignKey("tag.id"), primary_key=True)
)

category_tags = Table(
    'category_tags',
    Base.metadata,
    Column("category_id", Integer, ForeignKey("category.id"), primary_key=True),
    Column("tag_id", Integer, ForeignKey("tag.id"), primary_key=True)
)


class Unit(Base):
    __items__ = ["id", "name", "grams", "created_at", "updated_at"]
    __tablename__ = "unit"
    name = mapped_column(String(80), nullable=False, unique=True)
    grams = mapped_column(Float, default=0)


class Tag(Base):
    __items__ = ["id", "name", "created_at", "updated_at"]
    __tablename__ = "tag"
    name = mapped_column(String(80), nullable=False)
    categories: Mapped[List['Category']] = relationship(
        secondary=category_tags,
        back_populates="tags"
    )
    recipes: Mapped[List['Recipe']] = relationship(
        secondary=tags,
        back_populates="tags"
    )


class Ingredient(Base):
    __tablename__ = "ingredient"
    __items__ = [
        "id", "name", "energy", "fats",
        "carbs", "proteins", "fibres", "salt",
        "is_liquid", "density", "created_at", "updated_at"
    ]

    name = mapped_column(String(80), nullable=False)
    approx_per_piece = mapped_column(Float, default=0)  # g / piece

    energy = mapped_column(Float, default=0)  # kcal
    fats = mapped_column(Float, default=0)  # g / 100g
    carbs = mapped_column(Float, default=0)  # g / 100g
    proteins = mapped_column(Float, default=0)  # g / 100g
    fibres = mapped_column(Float, default=0)  # g / 100g
    salt = mapped_column(Float, default=0)  # g / 100g

    is_liquid = mapped_column(Boolean, default=False)
    density = mapped_column(Float, default=1000)  # g / L

    ingredients_items = relationship('IngredientItem', uselist=True, back_populates="ingredient")


class IngredientItem(Base):
    """
    -> [included in shopping list] <amount> <unit> <ingredient> (<note>)
    -> [x] 400 g tomatoes (roughly chopped)
    """
    __tablename__ = "ingredient_item"
    __items__ = ["id", "ingredient", "amount", "unit", "note", "order", "created_at", "updated_at"]

    ingredient_id = Column(Integer, ForeignKey(Ingredient.id))
    ingredient = relationship(Ingredient, uselist=False, back_populates="ingredients_items")
    amount = Column(Float, nullable=True, default=0)
    unit_id = Column(Integer, ForeignKey('unit.id'), nullable=True)
    unit = relationship("Unit")
    order = Column(Integer, default=0)

    note = Column(String(10), nullable=True)
    exclude = Column(Boolean(), default=False)  # todo: remove?

    recipes = relationship('Recipe', secondary=ingredients, back_populates="ingredients")


class Category(Base):
    __tablename__ = "category"
    __items__ = ["id", "name", "tags", "created_at", "updated_at"]
    name = Column(String(80), nullable=False)
    tags: Mapped[List[Tag]] = relationship(
        secondary=category_tags,
        back_populates="categories"
    )


class Recipe(Base):
    __tablename__ = "recipe"
    __items__ = [
        "id",
        "title",
        "subtitle",
        "ingredients",
        "favorite",
        "body",
        "source",
        "source_name",
        "tags",
        "portions",
        "detail_image",
        "thumbnail_image",
        "created_at",
        "updated_at"
    ]

    title = Column(String(80), nullable=False)
    subtitle = Column(String(50), nullable=True)
    source_name = Column(String(100), nullable=True)
    source = Column(String(100), nullable=True)
    draft = Column(Boolean(), default=False)
    favorite = Column(Boolean(), default=False)
    portions = Column(Integer, default=4)
    thumbnail_image = Column(String(100), nullable=True)
    detail_image = Column(String(100), nullable=True)
    ingredients: Mapped[List[IngredientItem]] = relationship(
        secondary=ingredients,
        cascade="all",
        order_by="IngredientItem.order"
    )
    tags = relationship(
        Tag,
        secondary=tags,
        lazy="subquery",
        back_populates="recipes"
    )

    # HTML content + rich text editor on the frontend
    body = Column(String(1000), nullable=True)



import os
from datetime import datetime

from fastapi import APIRouter, HTTPException
from fastapi import UploadFile
from loguru import logger

from chef.api.common import generic_get
from chef.controllers import RecipesController, CategoriesController
from chef.images import  CategoryHandler
from chef.settings import settings

recipes = RecipesController()
categories = CategoriesController()


router = APIRouter()


def _create_tmp_dir():
    upload_path = settings.upload_folder
    if not os.path.isdir(upload_path):
        raise HTTPException(status_code=500, detail="Server misconfiguration, invalid images path.")

    image_dir_name = f"chef_upload_{datetime.utcnow().timestamp()}"
    path = os.path.join(upload_path, image_dir_name)
    os.mkdir(path)
    return path


# @router.post('/recipes/{recipe_id}')
# async def post_recipe_image(recipe_id: int, image: UploadFile):
#     await generic_get(recipes, recipe_id)
#     logger.debug(f"uploading recipe (id={recipe_id}) image: {image.filename}, {image.size}b")
#
#     handler = Handler(image.file, recipe_id)
#     handler.create_images_set()
#
#     return f"Recipe images sent for processing...", 200
#
#
@router.post("/categories/{category_id}")
async def post_category_image(category_id, image: UploadFile):
    await generic_get(categories, category_id)

    logger.debug(f"uploading category (id={category_id}) image: {image.filename}, {image.size}b")
    handler = CategoryHandler(image.file, category_id)
    try:
        handler.create_images_set()
    except Exception as exc_info:
        logger.error("Image processing failed")
        logger.exception(exc_info)
        raise HTTPException(status_code=500, detail="Image processing failed") from exc_info

    return f"Images sent for processing...", 200



import asyncio
import glob
import os

from sqlalchemy.orm import Session

from chef.models import engine
from chef.settings import settings

from chef.controllers import RecipesController

recipes = RecipesController()


def __is_number(string: str):
    try:
        int(string.split("/")[-1])
        return True
    except:
        return False


def run():

    folders = glob.glob(settings.images_folder + "/*")
    recipe_folders = [f for f in folders if __is_number(f)]

    with Session(engine()) as db_session:
        for recipe_folder in recipe_folders:
            recipe_id = int(recipe_folder.split("/")[-1])
            image_path = os.path.join(recipe_folder, 'medium.jpeg')

            recipes.update_thumbnail(db_session, recipe_id, image_path)
            recipes.update_image(db_session, recipe_id, image_path)

        db_session.commit()


if __name__ == "__main__":
    run()



import os
from alembic.config import Config
from alembic import command

from loguru import logger

# alembic_config_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "alembic.ini"))

migrations_dir = os.path.abspath(os.path.dirname(__file__))


def run():
    alembic_cfg = Config()
    alembic_cfg.file_config["alembic"]["script_location"] = migrations_dir
    logger.info(f"Creating ad-hoc alembic.ini, {migrations_dir=}")

    logger.info(f"running the migrations >> head")
    command.upgrade(alembic_cfg, "head")

    logger.info(f"migrations completed")


if __name__ == "__main__":
    run()



import asyncio
from datetime import datetime

import httpx
from fastapi import APIRouter, HTTPException, UploadFile
from loguru import logger
from sqlalchemy.orm import Session

from chef.api.common import generic_get
from chef.controllers import RecipesController, IngredientsController
from chef.models import engine
from chef.schemas import Recipe, CreateOrUpdateRecipe, RecipeListItem
from chef.settings import settings

router = APIRouter()
recipes = RecipesController()
ingredients = IngredientsController()


# Store the current recipe id in memory
CURRENT_RECIPES_IDS = []


@router.get("")
async def get_recipes(
        category: int = None,
        favorite: bool = False,
        since: datetime = None,
) -> list[Recipe | RecipeListItem]:
    with Session(engine()) as session:
        if favorite:
            result = await recipes.get_all_and_filter(session, favorite=favorite)

        elif category:
            result = await recipes.get_by_category(session, category)
        elif since:
            result = await recipes.get_all_since(session, since=since)
        else:
            result = await recipes.get_all(session)

        return sorted(result, key=lambda rec: rec.title)


@router.get("/{item_id}")
async def get_recipe(item_id: int) -> Recipe:
    with Session(engine()) as session:
        return await recipes.get_single(session, item_id)


@router.post("/{item_id}/parse", response_model=None)
async def parse_recipe(item_id: int) -> any:
    with Session(engine()) as session:
        recipe = await recipes.get_single(session, item_id)

    async with httpx.AsyncClient() as client:
        response = await client.post(
            settings.gpt_enpoint_url,
            json={
                "message": recipe.body,
            },
            headers={"content_type": "application/json"},
            timeout=30.0
        )
        response.raise_for_status()
    parsed_ingredients = response.json()["response"]["ingredients"]

    return parsed_ingredients


@router.delete("/{item_id}")
async def delete_recipe(item_id: int) -> None:
    with Session(engine()) as session:
        await recipes.delete_single(session, item_id)


@router.post('', status_code=201)
async def create_recipe(create_data: CreateOrUpdateRecipe) -> Recipe:

    if any(not i.ingredient.name for i in create_data.ingredients):
        raise HTTPException(
            status_code=400,
            detail="Empty ingredient name not allowed.",
        )

    with Session(engine()) as session:
        return await recipes.create(session, create_data)


@router.put('/{item_id}')
async def update_recipe(item_id: int, update_data: CreateOrUpdateRecipe) -> Recipe:

    if any(not i.ingredient.name for i in update_data.ingredients):
        raise HTTPException(
            status_code=400,
            detail="Empty ingredient name not allowed."
        )

    with Session(engine()) as session:
        return await recipes.update(session, item_id, update_data)


@router.post('/{recipe_id}/thumbnail-image')
async def post_recipe_thumbnail(recipe_id: int, image: UploadFile):
    await generic_get(recipes, recipe_id)
    logger.debug(f"uploading thumbnail for recipe (id={recipe_id}) image: {image.filename}, {image.size / 1000}kB")
    with Session(engine()) as session:
        recipes.update_thumbnail(session, recipe_id, image.file)
        session.commit()

    return await generic_get(recipes, recipe_id)


@router.post('/{recipe_id}/detail-image')
async def post_recipe_image(recipe_id: int, image: UploadFile) -> Recipe:
    await generic_get(recipes, recipe_id)
    logger.debug(f"uploading image for recipe (id={recipe_id}) image: {image.filename}, {image.size / 1000}kB")
    with Session(engine()) as session:
        recipes.update_image(session, recipe_id, image.file)
        session.commit()

    return await generic_get(recipes, recipe_id)



from typing import List

from fastapi import APIRouter

from chef.api.common import generic_get, generic_get_all, generic_delete, generic_create
from chef.controllers import TagController
from chef.schemas import Tag, UpdateTag

router = APIRouter()
tags = TagController()


@router.get("")
async def get_tags() -> List[Tag]:
    return await generic_get_all(tags)


@router.get("/{item_id}")
async def get_tag(item_id: int) -> Tag:
    return await generic_get(tags, item_id)


@router.post("", status_code=201)
async def create_tag(tag_data: UpdateTag) -> Tag:
    return await generic_create(tags, tag_data)


@router.delete("/{item_id}")
async def delete_tag(item_id: int) -> None:
    return await generic_delete(tags, item_id)



"""Add fibres/proteins.

Revision ID: f91610b78d55
Revises: 3251a114750c
Create Date: 2021-04-10 11:41:22.149343

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'f91610b78d55'
down_revision = '3251a114750c'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('fibers', sa.Float(), nullable=True))
    op.add_column('ingredient', sa.Column('proteins', sa.Float(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('protein')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('protein', sa.FLOAT(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('proteins')
        batch_op.drop_column('fibers')
    # ### end Alembic commands ###



from typing import List

from fastapi import APIRouter

from chef.api.common import generic_get, generic_get_all, generic_delete, generic_update
from chef.controllers import IngredientsController
from chef.schemas import Ingredient, UpdateIngredient

router = APIRouter()
ingredients = IngredientsController()


@router.get("")
async def get_ingredients() -> List[Ingredient]:
    return await generic_get_all(ingredients)


@router.get("/{item_id}")
async def get_ingredient(item_id: int) -> Ingredient:
    return await generic_get(ingredients, item_id)


@router.delete("/{item_id}")
async def delete_ingredient(item_id: int) -> None:
    return await generic_delete(ingredients, item_id)


@router.put("/{item_id}")
async def update_ingredient(item_id, update_data: UpdateIngredient):
    return await generic_update(ingredients, item_id, update_data)
